@@ -14,7 +14,7 @@
 
 ## This core file sets up the load path and applies any necessary monkeypatches.
 
-raise "Sonic Pi requires Ruby 2+ to be installed. You are using version #{RUBY_VERSION}" if RUBY_VERSION < "2"
+raise "Sonic Pi requires Ruby 2.4+ to be installed. You are using version #{RUBY_VERSION}" if RUBY_VERSION < "2.4"
 
 ## Ensure native lib dir is available
 require 'rbconfig'
@@ -32,7 +32,6 @@ rescue LoadError
   warn "Non-critical error: Could not load did_you_mean"
 end
 
-require 'hamster/vector'
 require 'wavefile'
 
 os = case RUBY_PLATFORM
@@ -64,7 +63,6 @@ $:.unshift "#{File.expand_path("../vendor", __FILE__)}/sys-proctable-1.1.3/lib/#
 
 $:.unshift "#{File.expand_path("../rb-native", __FILE__)}/#{ruby_api}/"
 
-require 'win32/process' if os == :windows
 
 ## Add aubio native library to ENV if not present (the aubio library needs to be told the location)
 native_lib_path = File.expand_path("../../native/", __FILE__)
@@ -80,14 +78,20 @@ module SonicPi
     class ThreadLocal
       attr_reader :vars, :local_vars
 
-      def initialize(parent=nil, overrides={})
+      def initialize(parent=nil, overrides=nil)
         raise "ThreadLocal can only be initialized with nil or a parent ThreadLocal" unless parent.nil? || parent.is_a?(ThreadLocal)
         @parent_visible = true
 
-        if parent
-          @parent_vars = parent.vars.merge(overrides)
-          @vars = parent.vars.merge(overrides)
+        if parent && !parent.vars.empty?
+          if overrides
+            @parent_vars = parent.vars.merge(overrides)
+            @vars = parent.vars.merge(overrides)
+          else
+            @parent_vars = parent.vars.dup
+            @vars = parent.vars.dup
+          end
         else
+          overrides = {} unless overrides
           @parent_vars = overrides
           @vars = overrides
         end
@@ -165,13 +169,13 @@ module SonicPi
       end
 
       def self.inc_idx!(increment=1, init=0)
-        ridx = __thread_locals.get(:sonic_pi_spider_random_gen_idx) || init
+        ridx = __thread_locals.get(:sonic_pi_spider_random_gen_idx, init)
         __thread_locals.set :sonic_pi_spider_random_gen_idx, ridx + increment
         ridx
       end
 
       def self.dec_idx!(decrement=1, init=0)
-        ridx = __thread_locals.get(:sonic_pi_spider_random_gen_idx) || init
+        ridx = __thread_locals.get(:sonic_pi_spider_random_gen_idx, init)
         __thread_locals.set :sonic_pi_spider_random_gen_idx, ridx - decrement
         ridx
       end
@@ -226,22 +230,14 @@ module SonicPi
     end
 
     module TLMixin
-      # def next(*args)
-      #   tick(self.object_id.to_s.to_sym, *args)
-      # end
-
-      # def curr(*args)
-      #   look(self.object_id.to_s.to_sym, *args)
-      # end
-
       def tick(*args)
         idx = SonicPi::Core::ThreadLocalCounter.tick(*args)
-        self[idx]
+        self.ring[idx]
       end
 
       def look(*args)
         idx = SonicPi::Core::ThreadLocalCounter.look(*args)
-        self[idx]
+        self.ring[idx]
       end
     end
 
@@ -336,10 +332,13 @@ module SonicPi
     class NotThreadSafeError < StandardError ; end
 
 
-    class SPMap < Hamster::Hash
+    class SPMap
+      attr_reader :map
+
       def initialize(*args)
-        super
-        res = self.all? {|k, v| k.sp_thread_safe? && v.sp_thread_safe?}
+        @map = Hash[*args]
+        @map.freeze
+        res = @map.all? {|k, v| k.sp_thread_safe? && v.sp_thread_safe?}
         @sp_thread_safe = !!res
       end
 
@@ -348,12 +347,12 @@ module SonicPi
       end
 
       def sp_log_inspect
-        if self.empty?
+        if @map.empty?
           return "(map)"
         else
           s = String.new("(map ")
           longest_key = keys.to_a.map(&:to_s).map(&:size).sort[-1] + 2
-          self.each do |k, v|
+          @map.each do |k, v|
             if k.is_a?(Symbol)
               pk = "#{k.to_s}:".ljust(longest_key)
               s << "#{pk} #{v.inspect},\n       "
@@ -368,11 +367,11 @@ module SonicPi
       end
 
       def inspect
-        if self.empty?
+        if @map.empty?
           return "(map)"
         else
           s = String.new("(map ")
-          self.each do |k, v|
+          @map.each do |k, v|
             if k.is_a?(Symbol)
               pk = "#{k.to_s}:"
               s << "#{pk} #{v.inspect}, "
@@ -386,20 +385,358 @@ module SonicPi
         return s
       end
 
+      def <(other)
+        @map < other.map
+      end
+
+      def <=(other)
+        @map <= other.map
+      end
+
+
+      def ==(other)
+        other.map == @map
+      end
+
+      def >(other)
+        @map > other.map
+      end
+
+      def >=(other)
+        @map >= other.map
+      end
+
+      def [](key)
+        @map[key]
+      end
+
+      def any?(&block)
+        @map.any?(&block)
+      end
+
+      def assoc(o)
+        @map.assoc(o)
+      end
+
+      def compact
+        self.class.new(@map.compact)
+      end
+
+      def dig(*keys)
+        @map.dig(*keys)
+      end
+
+      def each(&block)
+        @map.each(&block)
+      end
+
+      def each_key(&block)
+        @map.each_key(&block)
+      end
+
+      def each_pair(&block)
+        @map.each_pair(&block)
+      end
+
+      def empty?
+        @map.empty?
+      end
+
+      def eql?(other)
+        @map.eql?(other)
+      end
+
+      def fetch(*args, &block)
+        @map.fetch(*args, &block)
+      end
+
+      def fetch_values(*args, &block)
+        @map.fetch_values(*args, &block)
+      end
+
+      def filter(&block)
+        self.class.new(@map.filter(&block))
+      end
+
+      def select(&block)
+        self.class.new(@map.select(&block))
+      end
+
+      def flatten(*args)
+        @map.flatten(*args)
+      end
+
+      def has_key?(k)
+        @map.has_key?(k)
+      end
+
+      def has_value?(v)
+        @map.has_value?(v)
+      end
+
+      def include?(k)
+        @map.include?(k)
+      end
+
+      def invert
+        self.class.new(@map.invert)
+      end
+
+      def key(v)
+        @map.key(v)
+      end
+
+      def key?(v)
+        @map.key?(v)
+      end
+
+      def keys
+        SPVector.new(@map.keys)
+      end
+
+      def length
+        @map.length
+      end
+
+      def size
+        @map.size
+      end
+
+      def merge(*hs, &block)
+        self.class.new(@map.merge(*hs, &block))
+      end
+
+      def rassoc(o)
+        SPVector.new(@map.rassoc(o))
+      end
+
+      def reject(&block)
+        self.class.new(@map.reject(&block))
+      end
+
+      def select(&block)
+        self.class.new(@map.select(&block))
+      end
+
+      def slice(*keys)
+        self.class.new(@map.slice(*keys))
+      end
+
+      def to_h
+        @map.dup
+      end
+
       def to_s
         inspect
       end
 
     end
 
-    class SPVector < Hamster::Vector
+    class SPVector
       include TLMixin
+      attr_reader :vec
+
       def initialize(list)
-        super
-        res = self.all? {|el| el.sp_thread_safe?}
+        @vec = list
+        @vec.freeze
+        res = @vec.all? {|el| el.sp_thread_safe?}
         @thread_safe = !!res
       end
 
+      def to_a
+        @vec.dup
+      end
+
+      alias :to_ary :to_a
+
+      def [](*idx)
+        if idx.size == 1
+          i = idx[0]
+          case i
+          when Range
+            res = @vec[i.min, i.max]
+            self.class.new(res) if res
+          else
+            @vec[map_index(i)]
+          end
+        else
+          res = @vec[*idx]
+          self.class.new(res) if res
+        end
+      end
+
+      def *(i)
+        self.class.new(@vec * i)
+      end
+
+      def &(v)
+          self.class.new(@vec & v)
+      end
+
+      def +(other)
+        case other
+        when SPVector
+          return self.class.new(@vec + other.vec)
+        when Array
+          return self.class.new(@vec + other)
+        else
+          o = other.to_f
+          return self.class.new(@vec.map{|el| el + o})
+        end
+      end
+
+      def -(other)
+        case other
+        when SPVector
+          return self.class.new(@vec - other.vec)
+        when Array
+          return self.class.new(@vec - other)
+        else
+          o = other.to_f
+          return self.class.new(@vec.map{|el| el - o})
+        end
+      end
+
+      def <=>(other)
+        @vec <=> other
+      end
+
+      def ==(other)
+        other.class == self.class && other.vec == @vec
+      end
+
+      def all?(&block)
+        @vec.all?(&block)
+      end
+
+      def any?(&block)
+        @vec.any?(&block)
+      end
+
+      def compact
+        self.class.new(@vec.compact)
+      end
+
+      def drop(n)
+        self.class.new(@vec.drop(n))
+      end
+
+      def drop_last(n=1)
+        self.class.new(@vec[0...(size-n)])
+      end
+
+      def each(&block)
+        @vec.each(&block)
+      end
+
+      def each_with_index(&block)
+        @vec.each_with_index(&block)
+      end
+
+      def empty?
+        @vec.empty?
+      end
+
+      def eql?(other)
+        other.class == self.class && @vec.eql?(other.vec)
+      end
+
+      def filter(&block)
+        self.class.new(@vec.select(&block))
+      end
+
+      def first(n=nil)
+        if n
+          self.class.new(@vec.first(n))
+        else
+          @vec.first
+        end
+      end
+
+      def flatten(*args)
+        self.class.new(@vec.flatten(*args))
+      end
+
+      def flat_map(&block)
+        self.class.new(@vec.flat_map(&block))
+      end
+
+      def index(*args, &block)
+        @vec.index(*args, &block)
+      end
+
+      def join(*args)
+        @vec.join(*args)
+      end
+
+      def last(n=nil)
+        if n
+          self.class.new(@vec.last(n))
+        else
+          @vec.last
+        end
+      end
+
+      def length
+        @vec.length
+      end
+
+      def map(&block)
+        @vec.map(&block)
+      end
+
+      def max(n=nil, &block)
+        if n
+          self.class.new(@vec.max(n, &block))
+        else
+          @vec.max(&block)
+        end
+      end
+
+      def min(n=nil, &block)
+        if n
+          self.class.new(@vec.min(n, &block))
+        else
+          @vec.min(&block)
+        end
+      end
+
+      def reverse
+        self.class.new(@vec.reverse)
+      end
+
+      def rotate(*args)
+        self.class.new(@vec.rotate(*args))
+      end
+
+      def choose
+        self[SonicPi::Core::SPRand.rand_i!(@vec.size)]
+      end
+
+      def sample
+        choose
+      end
+
+      def size
+        @vec.size
+      end
+
+      def shuffle
+        self.class.new(@vec.shuffle)
+      end
+
+      def sort(&block)
+        self.class.new(@vec.sort(&block))
+      end
+
+      def uniq(&block)
+        self.class.new(@vec.uniq(&block))
+      end
+
+      def values_at(*args)
+        self.class.new(@vec.values_at(*args))
+      end
+
       def sp_thread_safe?
         return @thread_safe
       end
@@ -410,70 +747,24 @@ module SonicPi
 
       def __sp_make_thread_safe
         return self if @thread_safe
-        map {|v| v.__sp_make_thread_safe }
-      end
-
-      def ___sp_preserve_vec_kind(a)
-        self.class.new(a)
+        self.class.new(map {|v| v.__sp_make_thread_safe })
       end
 
       def list_diff(other)
-        ___sp_preserve_vec_kind(self.to_a - other.to_a)
+        self.class.new(@vec.to_a - other.to_a)
       end
 
       def list_concat(other)
-        ___sp_preserve_vec_kind(self.to_a + other.to_a)
-      end
-
-      def -(other)
-        if other.is_a?(Array) || other.is_a?(SPVector)
-          return list_diff(other)
-        else
-          o = other.to_f
-          return self.map{|el| el - o}
-        end
-      end
-
-      def +(other)
-        if other.is_a?(Array) || other.is_a?(SPVector)
-          return list_concat(other)
-        else
-          o = other.to_f
-          return self.map{|el| el + o}
-        end
+        self.class.new(@vec.to_a + other.to_a)
       end
 
       def scale(val)
         val = val.to_f
-        return self.map{|el| el * val}
-      end
-
-      def [](idx, len=(missing_length = true))
-        return nil unless idx
-        raise InvalidIndexError, "Invalid index: #{idx.inspect}, was expecting a number or range" unless idx && (idx.is_a?(Numeric) || idx.is_a?(Range))
-        return nil if self.empty?
-        if idx.is_a?(Numeric) && missing_length
-          idx = map_index(idx)
-          super idx
-        else
-          if missing_length
-            super(idx)
-          else
-            super(idx, len)
-          end
-        end
-      end
-
-      def choose
-        self[SonicPi::Core::SPRand.rand_i!(self.size)]
-      end
-
-      def sample
-        choose
+        return self.class.new(@vec.map{|el| el * val})
       end
 
       def ring
-        if is_a?(SonicPi::Core::RingVector)
+        if is_a?(SonicPi::Core::SPVector)
           self
         else
           SonicPi::Core::RingVector.new(self)
@@ -507,10 +798,6 @@ module SonicPi
         self * n
       end
 
-      def drop_last(n=1)
-        self[0...(size-n)]
-      end
-
       def take_last(n=1)
         self if n >= size
         self[(size-n)..-1]
@@ -521,16 +808,16 @@ module SonicPi
       end
 
       def take(n)
-        return [].ring if n == 0
+        return self.class.new([]) if n == 0
         return self.reverse.take(-n) if n < 0
-        return super if n <= @size
-        self + take(n - @size)
+        return self.class.new([]) if @vec.size < 1
+        return self.class.new(@vec.take(n)) if n <= @vec.size
+        self.class.new(@vec.cycle.take(n))
       end
 
-      def drop(n)
-        return [].ring if n >= @size
-        super
-      end
+
+
+
 
       def pick(n=nil, *opts)
         # mangle args to extract nice behaviour
@@ -573,10 +860,6 @@ module SonicPi
         end
       end
 
-      def to_a
-        Array.new(self)
-      end
-
       def stretch(num_its)
         res = []
         self.each do |v|
@@ -584,7 +867,7 @@ module SonicPi
             res << v
           end
         end
-        ___sp_preserve_vec_kind(res)
+        self.class.new(res)
       end
 
       def map_index(idx)
@@ -640,6 +923,13 @@ module SonicPi
   end
 end
 
+class Time
+  def __sp_make_thread_safe
+    return self if frozen?
+    self.clone.freeze
+  end
+end
+
 
 class String
   def sp_thread_safe?
@@ -761,6 +1051,10 @@ class Array
     frozen? && self.all? {|el| el.sp_thread_safe?}
   end
 
+  def to_spv
+    SonicPi::Core::SPVector.new(self)
+  end
+
   def ring
     SonicPi::Core::RingVector.new(self)
   end
@@ -869,10 +1163,6 @@ class Hash
   end
 end
 
-
-
-# Meta-glasses from our hero Why to help us
-# see more clearly..
 class Object
 
   def sp_log_inspect
@@ -899,26 +1189,21 @@ class Object
     self.to_a.ring
   end
 
-  def next(*args)
-    self.to_a.next(*args)
-  end
-
-  def curr(*args)
-    self.to_a.curr(*args)
-  end
-
   def tick(*args)
-    self.to_a.tick(*args)
+    self.ring.tick(*args)
   end
 
   def look(*args)
-    self.to_a.look(*args)
+    self.ring.look(*args)
   end
 
+  # Meta-glasses from our hero Why to help us
+  # see more clearly..
+
   # The hidden singleton lurks behind everyone
   def metaclass; class << self; self; end; end
 
-  def meta_eval(&blk); metaclass.instance_eval &blk; end
+  def meta_eval(&blk); metaclass.instance_eval(&blk); end
   # Adds methods to a metaclass
   def meta_def name, &blk
     meta_eval { define_method name, &blk }
