@@ -36,6 +36,7 @@ module SonicPi
       @midi_osc_in_port = ports[:osc_midi_in_port]
       @midi_osc_out_port = ports[:osc_midi_out_port]
       @erlang_port = ports[:erlang_port]
+      @server_port = ports[:server_port]
       @msg_queue = msg_queue
       @error_occured_mutex = Mutex.new
       @error_occurred_since_last_check = false
@@ -60,12 +61,20 @@ module SonicPi
       init_or_reset_midi
     end
 
+    def __exec_path(path)
+      case os
+      when :windows
+        path
+      else
+        "exec #{path}"
+      end
+    end
 
     def __erl_mut_start_erlang
       return @erlang_pid if @erlang_pid
       # Start Erlang
       begin
-        erlang_cmd = "#{erlang_boot_path} -noshell -pz \"#{erlang_server_path}\" -s pi_server start #{@erlang_port}"
+        erlang_cmd = __exec_path("#{erlang_boot_path} -noshell -pz \"#{erlang_server_path}\" -s pi_server start #{@erlang_port} #{@osc_cues_port} #{@server_port}")
         STDOUT.puts erlang_cmd
         @erlang_pid = spawn erlang_cmd, out: erlang_log_path, err: erlang_log_path
         register_process(@erlang_pid)
@@ -677,7 +686,14 @@ module SonicPi
     def reb_mut_spawn_midi_m2o
       success = true
       begin
-        m2o_spawn_cmd = "'#{osmid_m2o_path}'" + " -b -o #{@midi_osc_in_port} -m 6 'Sonic Pi'"
+        osmid_osc_template = case os
+                       when :windows
+                         '/midi:$n:$i:$c/$m'
+                       else
+                         '/midi:\$n:\$i:\$c/\$m'
+                       end
+
+        m2o_spawn_cmd = __exec_path("'#{osmid_m2o_path}' -t #{osmid_osc_template} -b -o #{@midi_osc_in_port} -m 6 'Sonic Pi'")
         Kernel.puts "Studio - Spawning m2o with:"
         Kernel.puts "    #{m2o_spawn_cmd}"
         @m2o_pid = spawn(m2o_spawn_cmd, out: osmid_m2o_log_path, err: osmid_m2o_log_path)
@@ -696,7 +712,7 @@ module SonicPi
     def reb_mut_spawn_midi_o2m
       success = true
       begin
-        o2m_spawn_cmd = "'#{osmid_o2m_path}'" + " -L -b -i #{@midi_osc_out_port} -O #{@midi_osc_in_port} -m 6"
+        o2m_spawn_cmd = __exec_path("'#{osmid_o2m_path}'" + " -L -b -i #{@midi_osc_out_port} -O #{@midi_osc_in_port} -m 6")
         Kernel.puts "Studio - Spawning o2m with:"
         Kernel.puts "    #{o2m_spawn_cmd}"
         @o2m_pid = spawn(o2m_spawn_cmd, out: osmid_o2m_log_path, err: osmid_o2m_log_path)
@@ -735,6 +751,12 @@ module SonicPi
             desc = args.each_slice(3).reduce("") { |s, v| s += "#{v[2]}\n" }
             @msg_queue.push({:type => :midi_in_ports, :val => desc})
           end
+        elsif address.is_a?(String) && address.end_with?('active_sensing')
+          # Ignore Active Sensing MIDI messages.
+          # This message is intended to be sent repeatedly to tell the receiver
+          # that a connection is alive.
+          # A MIDI device sending these will send one every 300ms.
+          # They quickly full up the cue log.
         else
           @register_cue_event_lambda.call(Time.now, p, @midi_osc_server_thread_id, d, b, m, address, args , 0)
         end
